from email import policy
from email.utils import parseaddr, make_msgid, formatdate
from email.mime.text import MIMEText
from flask import Flask, request
from srht.config import cfg, cfgi
from string import Template
from urllib.parse import quote as urlquote
import email
import html.parser
import json
import os
import pygit2
import re
import smtplib

app = Flask(__name__)

smtp_host = cfg("mail", "smtp-host", default=None)
smtp_port = cfgi("mail", "smtp-port", default=None)
smtp_user = cfg("mail", "smtp-user", default=None)
smtp_password = cfg("mail", "smtp-password", default=None)

def render_template(templ, **kwargs):
    path = os.path.join(os.path.dirname(__file__), "replies", templ)
    with open(path) as f:
        tmpl = Template(f.read())
        return tmpl.substitute(**kwargs)

def send_reply(original, reply):
    reply["From"] = "git-send-email.io <devnull@sr.ht>"
    reply["To"] = original["From"]
    reply["Cc"] = original["To"]
    reply["Date"] = formatdate()
    if reply.get("Message-ID") is None:
        reply["Message-ID"] = make_msgid()
    reply["In-Reply-To"] = original["Message-ID"]
    reply["Subject"] = "Re: " + (
            original.get("Subject") or "Your email to git-send-email.io")
    print(reply.as_string(unixfrom=True))
    sender = parseaddr(original["From"])
    recipients = [sender[1]]
    if reply["Cc"]:
        recipients.append(parseaddr(reply["Cc"])[1])
    smtp = smtplib.SMTP(smtp_host, smtp_port)
    smtp.ehlo()
    smtp.starttls()
    smtp.login(smtp_user, smtp_password)
    smtp.sendmail(smtp_user, recipients, reply.as_string(unixfrom=True))
    smtp.quit()

patch_subject = re.compile(r"\[PATCH( v[0-9]+)?] ")
stage_1 = re.compile(r"I['’]?m about to try git[ -]send[ -]?email[!.]?",
        flags=re.IGNORECASE)
stage_2 = re.compile(r"I have successfully used git[ -]send[ -]?email[!.]?",
        flags=re.IGNORECASE)

def get_reply(diff, mail):
    if patch_subject.match(mail["Subject"]) is None:
        # Crude way of filtering out a patch series
        return None
    from_ = parseaddr(mail["From"])
    if not diff:
        return MIMEText(render_template("not-a-patch", **{"from": from_[0]}))
    patches = list(diff)
    if len(patches) != 1:
        return MIMEText(render_template("too-many-files", **{"from": from_[0]}))
    patch = patches[0]
    if patch.delta.status_char() != 'A':
        return MIMEText(render_template("wrong-changes", **{"from": from_[0]}))
    # Note: there will only ever be one since we asserted that this is an
    # addition:
    hunk = patch.hunks[0]
    if hunk.new_lines != 1:
        return MIMEText(render_template("wrong-content", **{"from": from_[0]}))
    content = hunk.lines[0].content
    if stage_1.match(content) is not None:
        msgid = make_msgid()
        reply = MIMEText(render_template("stage-1-success", **{
            "from": from_[0],
            "new_file_path": patch.delta.new_file.path,
            "content": content.rstrip(),
            "msgid": msgid,
            "msgid_url": urlquote(msgid),
        }))
        reply["Message-ID"] = msgid
        return reply
    elif stage_2.match(content) is not None:
        return MIMEText(render_template("stage-2-success", **{"from": from_[0]}))
    else:
        return MIMEText(render_template("wrong-content", **{"from": from_[0]}))
    return None

@app.route("/webhook", methods=["POST"])
def webhook():
    payload = json.loads(request.data.decode('utf-8'))
    envelope = str(payload["envelope"])
    mail = email.message_from_string(envelope,
            policy=email.policy.SMTP)
    body = None
    for part in mail.walk():
        if part.is_multipart():
            continue
        content_type = part.get_content_type()
        [charset] = part.get_charsets("utf-8")
        if content_type == 'text/plain':
            body = part.get_payload(decode=True).decode(charset)
            break
    if not body:
        # ???
        return "???"
    try:
        diff = pygit2.Diff.parse_diff(body.replace("\r\n", "\n"))
    except:
        diff = None
    reply = get_reply(diff, mail)
    if reply:
        send_reply(mail, reply)
    else:
        print("no response necessary")
    return "thanks!"
